package com.miya.manning.vo.scripts;


import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.io.*;
import java.util.ResourceBundle;

public class SSHScript implements ExcuteableScript {

    private String params;

    private static final Logger log = Logger.getLogger(SSHScript.class);


    public SSHScript() {
    }

    ;

    public SSHScript(String params) {
        this.params = params;
    }


    public String getParams() {
        return params;
    }

    public void setParams(String params) {
        this.params = params;
    }

    @Override
    public void execute(String params) {
        Connection conn = SSHScript.login();
        String result = "";
        try{
            if(conn != null){
                Session session = conn.openSession();//打开一个会话
                session.execCommand(params);
                result = processStdout(session.getStdout(),"UTF-8");
                if(StringUtils.isBlank(result)){
                    log.error("得到的标准输出为空,链接conn:" + conn + "执行的命令:" + params);
                    result = processStdout(session.getStderr(),"UTF-8");
                }else {
                    log.info("执行命令成功！链接conn:" + conn + "执行的命令:" + params + ",结果为：" + result);
                }
                conn.close();
                session.close();

            }
        }catch (IOException e){
            log.error("执行脚本失败,链接conn:" + conn + "执行的命令:"+ params + "  " + e.getMessage() );
        }

    }


    public static Connection login() {
        Connection conn = null;
        ResourceBundle bundle = ResourceBundle.getBundle("config");
        String ip = bundle.getString("ssh_ip");
        int port = Integer.parseInt(bundle.getString("ssh_port"));
        String username = bundle.getString("ssh_username");
        String password = bundle.getString("ssh_password");
        try {
            conn = new Connection(ip,port);
            conn.connect();
            boolean flag = conn.authenticateWithPassword(username, password);
            if (flag) {
                log.info("登录成功！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("登录失败！");
        }
        return conn;
    }

    private static String processStdout(InputStream in, String charset){
        InputStream  stdout = new StreamGobbler(in);
        StringBuffer buffer = new StringBuffer();;
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(stdout,charset));
            String line = null;
            while((line=br.readLine()) != null){
                buffer.append(line +"\n");
            }
        } catch (UnsupportedEncodingException e) {
            log.error("解析脚本出错:" +e.getMessage());
            e.printStackTrace();
        } catch (IOException e) {
            log.error("解析脚本出错:"+e.getMessage());
            e.printStackTrace();
        }
        return buffer.toString();
    }
}
